/*
 * @brineSamplePlot.java Version 1.1 06/13/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.plot;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

import brine.brineStruct;           // Raw Brine Data
import brine.brineListStruct;       // Raw Brine Data List
import brine.plot.brinePlotStruct;  // Common Brine Data Sample
import brine.plot.brineStiffPlot;   // Stiff Plot
import brine.plot.brineCollinsPlot; // Collins Bar Diagram

/** CLASS brineSamplePlot
 *  <p> This Class will create the Stiff Diagram Plot.
 *
 *  @version 1.1 06/13/2011
 *  @author  John R. Victorine
 */

public class brineSamplePlot extends Canvas
{
  private int             iRow    = 0;
  private brineListStruct stBrine = null; // Common Brine Data
  private brinePlotListStruct st  = null; // Plot Brine Data List

  private brineStiffPlot   plotStiff   = null; // Stiff Plot Diagram
  private brineCollinsPlot plotCollins = null; // Collins Diagram

  // Brine Math Variables

  private static final int _CATIONS  = brine.math.brineMath._CATIONS;
  private static final int _ANIONS   = brine.math.brineMath._ANIONS;

  private static final int CATIONS[] = brine.math.brineMath.CATIONS;
  private static final int ANIONS[]  = brine.math.brineMath.ANIONS;

  private static final int _COLUMNS = 15;
  private static final int _TOTAL   = 18;

  private String sAnalysis[] = { "", "", "",
                                 "", "", "",
                                 "", "", "",
                                 "", "", "",
                                 "", "", "",
                                 "", "", "" };

  // X Plot Position

  private static final int _HEADER = 200;

  private static final int _TEXT   =  20;
  private static final int _TEXT2  = 290;
  private static final int _TEXT3  = 520;

  private static final int _UNIT   =  10;

  private static final int _CA     =  60;
  private static final int _MG     = 115;
  private static final int _NA     = 170;
  private static final int _K      = 225;
  private static final int _FE     = 280;
  private static final int _MN     = 335;

  private static final int _CL     = 390; //280;
  private static final int _BR     = 445; //335;
  private static final int _I      = 500; //390;
  private static final int _SO4    = 555; //445;
  private static final int _HCO3   = 610; //500;
  private static final int _CO3    = 665; //555;
  private static final int _PO4    = 720; //555;

  // Y Plot Position

  private static final int _TOP     =  20; // Brine Sample Plot
  private static final int _LINE_0  =  40; // Brine Record
  private static final int _LINE_1  =  60;
  private static final int _LINE_2  =  80;
  private static final int _LINE_3  = 100;
  private static final int _LINE_4  = 120;

  private static final int _LINE_5  = 140; // Source

  private static final int _LINE_6  = 160; // Table Header
  private static final int _LINE_7  = 180; // Table - mg/l
  private static final int _LINE_8  = 200; // Table - meq/l
  private static final int _LINE_9  = 220; // Table - % meq/l

  private static final int _LINE_10 = 510; // General Analysis
  private static final int _LINE_11 = 530; // 1st Line
  private static final int _LINE_12 = 550; // 2nd Line
  private static final int _LINE_13 = 570; // 3rd Line
  private static final int _LINE_14 = 590; // 4th Line
  private static final int _LINE_15 = 610; // 5th Line
  private static final int _LINE_16 = 630; // 6th Line

  // Stiff Plot Position

  private static final int _X_STIFF = 10;
  private static final int _Y_STIFF = 290;

  // Collins Bar Plot Position

  private static final int _X_COLLINS = 325;
  private static final int _Y_COLLINS = 230;

  // Plot Dimensions

//  private int iWidth  = 625;  // Width of Plot
  private int iWidth  = 775;  // Width of Plot
  private int iHeight = 700; //650;  // Height of Plot with the Titles

  /** CONSTRUCTOR brineSamplePlot()
   *  <p> This is the constructor for this class.
   *  @param stBrine = Brine Data List Structure
   */

  public brineSamplePlot( brineListStruct stBrine )
  {
    this.stBrine = stBrine;
    setBrineData();
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
	if (st != null)
	  st.delete();
	st = null;

	stBrine = null;
  }

  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setBrineData()
   * <p> This method will set the Brine Data List Structure
   */

  public void setBrineData()
  {
    int i,j;
    int lenAnions  = 0;
    int lenCations = 0;

	st = new brinePlotListStruct();

	if (stBrine != null)
	{
	  lenAnions  = brine.math.brineMath.getRows( _ANIONS );
	  lenCations = brine.math.brineMath.getRows( _CATIONS );

	  st         = new brinePlotListStruct();
	  st.iCount  = stBrine.iCount;
	  st.stItem = new brinePlotStruct[st.iCount];

	  for (i=0; i<stBrine.iCount; i++)
	  {
	    st.stItem[i] = new brinePlotStruct();

        st.stItem[i].mgl_cations      = new double[lenCations];
        st.stItem[i].meql_cations     = new double[lenCations];
        st.stItem[i].per_meql_cations = new double[lenCations];

        st.stItem[i].mgl_anions       = new double[lenAnions];
        st.stItem[i].meql_anions      = new double[lenAnions];
        st.stItem[i].per_meql_anions  = new double[lenAnions];
	  }

	  for (i=0; i<stBrine.iCount; i++)
	  {
		st.stItem[i].depthStart  = stBrine.stItem[i].dTOP;
		st.stItem[i].depthEnd    = stBrine.stItem[i].dBASE;

	    st.stItem[i].mgl_cations =
	      brine.math.brineMath.get_mg_per_liter( _CATIONS, i, stBrine );

	    st.stItem[i].mgl_anions =
	      brine.math.brineMath.get_mg_per_liter( _ANIONS, i, stBrine );

        st.stItem[i].meql_cations =
          brine.math.brineMath.mg_to_meq_per_liter(
  		    _CATIONS, st.stItem[i].mgl_cations );

        st.stItem[i].meql_anions =
          brine.math.brineMath.mg_to_meq_per_liter(
	  	      _ANIONS, st.stItem[i].mgl_anions );

        st.stItem[i].per_meql_cations =
          brine.math.brineMath.percent_meq_per_liter(
		      _CATIONS, st.stItem[i].meql_cations );

        st.stItem[i].per_meql_anions =
          brine.math.brineMath.percent_meq_per_liter(
		      _ANIONS, st.stItem[i].meql_anions );
      }

      setAnalysisSection();
	}
  }

  /** Method setAnalysisSection( brineListStruct stBrine )
   * <p> This method will set the Brine Data List Structure
   */

  public void setAnalysisSection()
  {
	int         icnt = 0;
	brineStruct stA  = null;

	if (stBrine != null)
	{
	  if (stBrine.iCount > 0)
	  {
		stA = stBrine.stItem[0];
		if (stA != null)
		{
	      if ((stBrine._FLUID > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Sample Fluid: " + stA.dFLUID +
		                                 " " + stA.sFLUID);
		    icnt++;
		  }

	      if ((stBrine._PH > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("PH: " + stA.dPH);
		    icnt++;
		  }

	      if ((stBrine._SPGR > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Specific Gravity: " + stA.dSPGR +
		                                 " " + stA.sSPGR);
		    icnt++;
		  }

	      if ((stBrine._SIGMA > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Specific Conductivity: " + stA.dSIGMA +
		                                 " " + stA.sSIGMA);
		    icnt++;
		  }

	      if ((stBrine._OHM > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Resistivity: " + stA.dOHM +
		                                 " " + stA.sOHM);
		    icnt++;
		  }

	      if ((stBrine._OHM75 > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Resistivity at 75 deg F: " + stA.dOHM75 +
		                                 " " + stA.sOHM75);
		    icnt++;
		  }

	      if ((stBrine._OHME > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Estimate Rw: " + stA.dOHME +
		                                 " " + stA.sOHME);
		    icnt++;
		  }

	      if ((stBrine._mV > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Redox Potential: " + stA.dmV +
		                                 " " + stA.smV);
		    icnt++;
		  }

	      if ((stBrine._DEG > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Temperature: " + stA.dDEG + " degrees F");
		    icnt++;
		  }

	      if ((stBrine._DEGF > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Temperature: " + stA.degF + " degrees F");
		    icnt++;
		  }

	      if ((stBrine._DEGC > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Temperature: " + stA.degC + " degrees C");
		    icnt++;
		  }

	      if ((stBrine._DEGi > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Initial Temperature: " + stA.degi +
		                                 " " + stA.sDegi);
		    icnt++;
		  }

	      if ((stBrine._DEGf > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Final Temperature: " + stA.degf +
		                                 " " + stA.sDegf);
		    icnt++;
		  }

	      if ((stBrine._DEGa > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Average Temperature: " + stA.dega +
		                                 " " + stA.sDega);
		    icnt++;
		  }

	      if ((stBrine._PSI > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Pressure: " + stA.dPSI +
		                                 " " + stA.sPSI);
		    icnt++;
		  }

	      if ((stBrine._PSIi > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Initial Pressure: " + stA.dPSIi +
		                                 " " + stA.sPSIi);
		    icnt++;
		  }

	      if ((stBrine._PSIf > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Final Pressure: " + stA.dPSIf +
		                                 " " + stA.sPSIf);
		    icnt++;
		  }

	      if ((stBrine._NTU > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Turbidity: " + stA.dNTU +
		                                 " " + stA.sNTU);
		    icnt++;
		  }

	      if ((stBrine._SALNTY > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Salinity: " + stA.dSALNTY +
		                                 " " + stA.sSALNTY);
		    icnt++;
		  }

	      if ((stBrine._HARDNESS > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Hardness: " + stA.dHARDNESS);
		    icnt++;
		  }

	      if ((stBrine._RATIO > -1) && (icnt < _COLUMNS))
	      {
		    sAnalysis[icnt] = new String("Anion/Cation Ratio: " + stA.dRATIO);
		    icnt++;
		  }

          icnt = _COLUMNS;
		  if (stBrine._TDS > -1)
		  {
		    sAnalysis[icnt] = new String("Total Dissolved Solids: " + stA.dTDS +
		                                 " " + stA.sTDS);
		    icnt++;
		  }

		  if (stBrine._TDSe > -1)
		  {
		    sAnalysis[icnt] = new String("Est. Total Solids: " + stA.dTDSe +
		                                 " " + stA.sTDSe);
		    icnt++;
		  }

		  if (stBrine._TDSc > -1)
		  {
		    sAnalysis[icnt] = new String("Computed Total Solids: " + stA.dTDSc +
		                                 " " + stA.sTDSc);
		    icnt++;
		  }
	    }
	  }
	}
  }

  /** Method setRow()
   * <p> This method will set the plot data to plot
   * @param iRow = The Data Row
   */

  public void setRow(int iRow)
  {
    this.iRow = iRow;

    plotStiff = new brineStiffPlot( st.stItem[iRow],
                                    _X_STIFF, _Y_STIFF );
    plotCollins = new brineCollinsPlot( st.stItem[iRow] );
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method draw()
   * <p> This method will draw the stiff diagram
   *  @param g - Graphics pointer.
   */

  public void draw(Graphics g)
  {
	String sLabel = "";
	int    length = 0;

    Font        fsb  = new Font("Monospaced", Font.BOLD, 14);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    g.setColor(Color.black);

    sLabel = new String( "Brine Sample Plot" );
    length = sLabel.length();
    g.drawString( sLabel, iWidth/2 - length*4 , _TOP );

    fsb  = new Font("Monospaced", Font.BOLD, 12);
    g.setFont( fsb );

    if (stBrine != null)
    {
	  if (stBrine.stItem[iRow] != null)
	  {
        if (stBrine.stItem[iRow].sName.length() > 0)
        {
          sLabel = new String( stBrine.stItem[iRow].sName );
          if (stBrine.stItem[iRow].sAPI.length() > 0)
            sLabel = new String( sLabel + " (" + stBrine.stItem[iRow].sAPI + ")" );
          sLabel = new String( "Well: " + sLabel );
          length = sLabel.length();
          g.drawString( sLabel, iWidth/2 - length*4 , _LINE_0 );
        }

        g.drawString( stBrine.stItem[iRow].sLab,        _TEXT, _LINE_1 );
        g.drawString( stBrine.stItem[iRow].sAddress,    _TEXT, _LINE_2 );
        g.drawString( stBrine.stItem[iRow].sCity + ", " +
                      stBrine.stItem[iRow].state_lab,   _TEXT, _LINE_3 );
        g.drawString( stBrine.stItem[iRow].zipcode,     _TEXT, _LINE_4 );

        g.drawString( "Sample Date: " +
                      stBrine.stItem[iRow].sRecovery,   _TEXT2, _LINE_4 );

        g.drawString( "Formation: " +
                      stBrine.stItem[iRow].sFORM,       _TEXT3, _LINE_1 );

        sLabel = new String( "" + stBrine.stItem[iRow].dTOP );
        if (stBrine.stItem[iRow].dBASE > 0.0)
          sLabel = new String( sLabel +
                               " - " + stBrine.stItem[iRow].dBASE );
        g.drawString( "Depth:     " +  sLabel,          _TEXT3, _LINE_2 );
//        g.drawString( "Latitude:  " +
//                      stBrine.stItem[iRow].dLatitude,   _TEXT2, _LINE_3 );
//        g.drawString( "Longitude: " +
//                      stBrine.stItem[iRow].dLongitude,  _TEXT2, _LINE_4 );

        plotTable( g );

        drawDiagrams( g );

        fsb  = new Font("Monospaced", Font.BOLD, 12);
        g.setFont( fsb );

        g.setColor(Color.black);

        drawGeneralAnalysis( g );
      }
    }
  }

  /** Method drawGeneralAnalysis()
   * <p> This method will draw the General Analysis Section
   *  @param g - Graphics pointer.
   */

  private void drawGeneralAnalysis(Graphics g)
  {
	String sLabel = "";
	int    length = 0;

	int    icnt   = 0;
	int    lne    = _LINE_10;
	int    ltxt   = _TEXT;

    Font        fsb  = new Font("Monospaced", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    if (stBrine != null)
    {
	  if (stBrine.stItem[iRow] != null)
	  {
        sLabel = new String( "General Analysis" );
        length = sLabel.length();
        g.drawString( sLabel, iWidth/2 - length*4 , _LINE_10 );

        for (int i=0; i<_TOTAL; i++)
        {
		  if (icnt == 0)
		  {
			ltxt = _TEXT;
            lne  = lne + 20;
		  }
		  if (icnt == 1)
		  {
			ltxt = _TEXT2;
		  }
		  if (icnt == 2)
		  {
			ltxt = _TEXT3;
		  }

          g.drawString( sAnalysis[i], ltxt, lne );
          icnt++;

          if (icnt == 3) { icnt = 0; }
		}
      }
    }
  }

  /** Method plotTable( Graphics g )
   *  <p> This method will paint the analyte Data Table
   */

  public void plotTable( Graphics g )
  {
	String sLabel = "";
	int    length = 0;
    double data   = 0.0;

    Font        fsb  = new Font("Monospaced", Font.BOLD, 11);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    g.setColor(Color.black);
    if (stBrine.stItem[iRow].source.length() > 0)
    {
      sLabel = new String( "Source: " +
                           stBrine.stItem[iRow].source.length() );
      length = sLabel.length();
      g.drawString( sLabel, iWidth/2 - length*4, _LINE_5 );
    }

    g.setColor(new Color( 0, 180, 0 ));
    g.drawLine(0, _LINE_6-13, iWidth, _LINE_6-13);

    g.setColor(Color.black);

    g.drawString( " Unit ",  _UNIT, _LINE_6 );

    g.drawString( "Ca    ",  _CA,   _LINE_6 );
    g.drawString( "Mg    ",  _MG,   _LINE_6 );
    g.drawString( "Na    ",  _NA,   _LINE_6 );
    g.drawString( "K     ",  _K,    _LINE_6 );
    g.drawString( "Fe    ",  _FE,   _LINE_6 );
    g.drawString( "Mn    ",  _MN,   _LINE_6 );

    g.drawString( "Cl    ",  _CL,   _LINE_6 );
    g.drawString( "Br    ",  _BR,   _LINE_6 );
    g.drawString( "I     ",  _I,    _LINE_6 );
    g.drawString( "SO4   ",  _SO4,  _LINE_6 );
    g.drawString( "HCO3  ",  _HCO3, _LINE_6 );
    g.drawString( "CO3   ",  _CO3,  _LINE_6 );
    g.drawString( "nPO4  ",  _PO4,  _LINE_6 );

    g.setColor(new Color( 0, 180, 0 ));
    g.drawLine(0, _LINE_6+7, iWidth, _LINE_6+7);
    g.drawLine(_CA-4, _LINE_6-13, _CA-4, _LINE_9+2);
    g.drawLine(_CL-4, _LINE_6-13, _CL-4, _LINE_9+2);


    fsb  = new Font("Monospaced", Font.BOLD, 10);
    g.setFont( fsb );

    g.setColor(Color.black);

    g.drawString( "   mg/l",  _UNIT, _LINE_7 );

    if (st.stItem[iRow].mgl_cations[0] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_cations[0], _CA, _LINE_7 );
    if (st.stItem[iRow].mgl_cations[1] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_cations[1], _MG, _LINE_7 );
    if (st.stItem[iRow].mgl_cations[2] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_cations[2], _NA, _LINE_7 );
    if (st.stItem[iRow].mgl_cations[3] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_cations[3], _K,  _LINE_7 );
    if ((st.stItem[iRow].mgl_cations[4] > 0.0) ||
        (st.stItem[iRow].mgl_cations[5] > 0.0))
    {
	  data = st.stItem[iRow].mgl_cations[4] + st.stItem[iRow].mgl_cations[5];
      g.drawString( "" + data, _FE,  _LINE_7 );
    }
    if ((st.stItem[iRow].mgl_cations[6] > 0.0) ||
        (st.stItem[iRow].mgl_cations[7] > 0.0))
    {
	  data = st.stItem[iRow].mgl_cations[6] + st.stItem[iRow].mgl_cations[7];
      g.drawString( "" + data, _MN,  _LINE_7 );
    }

    if (st.stItem[iRow].mgl_anions[0] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_anions[0], _CL,   _LINE_7 );
    if (st.stItem[iRow].mgl_anions[1] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_anions[1], _BR,   _LINE_7 );
    if (st.stItem[iRow].mgl_anions[2] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_anions[2], _I,    _LINE_7 );
    if (st.stItem[iRow].mgl_anions[3] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_anions[3], _SO4,  _LINE_7 );
    if (st.stItem[iRow].mgl_anions[4] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_anions[4], _HCO3, _LINE_7 );
    if (st.stItem[iRow].mgl_anions[5] > 0.0)
      g.drawString( "" + st.stItem[iRow].mgl_anions[5], _CO3,  _LINE_7 );
    if ((st.stItem[iRow].mgl_anions[6] > 0.0) ||
        (st.stItem[iRow].mgl_anions[7] > 0.0) ||
        (st.stItem[iRow].mgl_anions[8] > 0.0))
    {
	  data = st.stItem[iRow].mgl_anions[6] +
	         st.stItem[iRow].mgl_anions[7] +
	         st.stItem[iRow].mgl_anions[8];
      g.drawString( "" + data, _PO4,  _LINE_7 );
	}

    g.drawString( "  meq/l",  _UNIT, _LINE_8 );

    if (st.stItem[iRow].meql_cations[0] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_cations[0], _CA, _LINE_8 );
    if (st.stItem[iRow].meql_cations[1] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_cations[1], _MG, _LINE_8 );
    if (st.stItem[iRow].meql_cations[2] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_cations[2], _NA, _LINE_8 );
    if (st.stItem[iRow].meql_cations[3] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_cations[3], _K,  _LINE_8 );
    if ((st.stItem[iRow].meql_cations[4] > 0.0) ||
        (st.stItem[iRow].meql_cations[5] > 0.0))
    {
	  data = st.stItem[iRow].meql_cations[4] + st.stItem[iRow].meql_cations[5];
      g.drawString( "" + data, _FE,  _LINE_8 );
	}
    if ((st.stItem[iRow].meql_cations[6] > 0.0) ||
        (st.stItem[iRow].meql_cations[7] > 0.0))
    {
	  data = st.stItem[iRow].meql_cations[6] + st.stItem[iRow].meql_cations[7];
      g.drawString( "" + data, _MN,  _LINE_8 );
    }

    if (st.stItem[iRow].meql_anions[0] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_anions[0], _CL,   _LINE_8 );
    if (st.stItem[iRow].meql_anions[1] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_anions[1], _BR,   _LINE_8 );
    if (st.stItem[iRow].meql_anions[2] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_anions[2], _I,    _LINE_8 );
    if (st.stItem[iRow].meql_anions[3] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_anions[3], _SO4,  _LINE_8 );
    if (st.stItem[iRow].meql_anions[4] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_anions[4], _HCO3, _LINE_8 );
    if (st.stItem[iRow].meql_anions[5] > 0.0)
      g.drawString( "" + st.stItem[iRow].meql_anions[5], _CO3,  _LINE_8 );
    if ((st.stItem[iRow].meql_anions[6] > 0.0) ||
        (st.stItem[iRow].meql_anions[7] > 0.0) ||
        (st.stItem[iRow].meql_anions[8] > 0.0))
    {
	  data = st.stItem[iRow].meql_anions[6] +
	         st.stItem[iRow].meql_anions[7] +
	         st.stItem[iRow].meql_anions[8];
      g.drawString( "" + data, _PO4,  _LINE_8 );
	}

    g.drawString( "% meq/l",  _UNIT, _LINE_9 );

    if (st.stItem[iRow].per_meql_cations[0] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_cations[0], _CA, _LINE_9 );
    if (st.stItem[iRow].per_meql_cations[1] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_cations[1], _MG, _LINE_9 );
    if (st.stItem[iRow].per_meql_cations[2] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_cations[2], _NA, _LINE_9 );
    if (st.stItem[iRow].per_meql_cations[3] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_cations[3], _K,  _LINE_9 );
    if ((st.stItem[iRow].per_meql_cations[4] > 0.0) ||
        (st.stItem[iRow].per_meql_cations[5] > 0.0))
    {
	  data = st.stItem[iRow].per_meql_cations[4] +
	         st.stItem[iRow].per_meql_cations[5];
      g.drawString( "" + data, _FE,  _LINE_9 );
    }
    if ((st.stItem[iRow].per_meql_cations[6] > 0.0) ||
        (st.stItem[iRow].per_meql_cations[7] > 0.0))
    {
	  data = st.stItem[iRow].per_meql_cations[6] +
	         st.stItem[iRow].per_meql_cations[7];
      g.drawString( "" + data, _MN,  _LINE_9 );
    }

    if (st.stItem[iRow].per_meql_anions[0] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_anions[0], _CL,   _LINE_9 );
    if (st.stItem[iRow].per_meql_anions[1] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_anions[1], _BR,   _LINE_9 );
    if (st.stItem[iRow].per_meql_anions[2] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_anions[2], _I,    _LINE_9 );
    if (st.stItem[iRow].per_meql_anions[3] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_anions[3], _SO4,  _LINE_9 );
    if (st.stItem[iRow].per_meql_anions[4] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_anions[4], _HCO3, _LINE_9 );
    if (st.stItem[iRow].per_meql_anions[5] > 0.0)
      g.drawString( "" + st.stItem[iRow].per_meql_anions[5], _CO3,  _LINE_9 );
    if ((st.stItem[iRow].per_meql_anions[6] > 0.0) ||
        (st.stItem[iRow].per_meql_anions[7] > 0.0) ||
        (st.stItem[iRow].per_meql_anions[8] > 0.0))
    {
	  data = st.stItem[iRow].per_meql_anions[6] +
	         st.stItem[iRow].per_meql_anions[7] +
	         st.stItem[iRow].per_meql_anions[8];
      g.drawString( "" + data, _PO4,  _LINE_9 );
	}

    g.setColor(new Color( 0, 180, 0 ));
    g.drawLine(0, _LINE_9+2, iWidth, _LINE_9+2);
  }

  /** Method drawDiagrams()
   * <p> This method will draw the stiff diagram
   *  @param g - Graphics pointer.
   */

  public void drawDiagrams(Graphics g)
  {
    plotStiff.draw( g, _X_STIFF, _Y_STIFF );
    plotCollins.draw( g, _X_COLLINS, _Y_COLLINS );
  }

  /** Method paint( Graphics g )
   * <p> This method will paint the stiff diagram Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
    draw(g);
  }
}

/*
 *  @version 1.1 06/13/2011
 *  @author  John Victorine
 */
